<%--
 * Component: dotcom
 *
 * Description:
 *    Correcting studies reside on multiple HUB servers (do the actual correction)
 *
 *
 *  @param cStudy                  Study Instance UID of the study to be corrected
 *  @param cNumDevs                Number of HUB servers the study resides on
 *  @param cTarget                 Device ID of the HUB server the study to be remain on
 *  @param cKeepImages             Device ID(s) of HUB server(s) having images to be moved to target HUB
 *  @param cKeepReport             Device ID(s) of HUB server(s) having reports to be moved to target HUB
 *  @param cOp                     Do the correction if value is 'correct', give a confirmation page otherwise
 * [@param cBack      Back url]
 * [@param cSession   Session id for caching study information collected from the HUB servers]
 *
--%><%@
	page import="java.io.*, java.util.*, imagemed.exception.*, imagemed.imsession.*, imagemed.jsputil.*, imagemed.util.*,
			imagemed.dotcom.Dotcom, imagemed.dotcom.multihub.*, imagemed.types.string.IMTypePN, java.text.Format,
			imagemed.dotcom.ObsoleteStudyTraversal"

%><%@
	page errorPage="../main/exceptionHandler.jsp"

%><%!

	/** Tab page (display page under this tab */
	static String m_tabPage = "cases/index.jsp";

%><%

	IMSession is = new IMSession( request, response, out );

	String parStudy = request.getParameter( "cStudy" );
	String parSession = request.getParameter( "cSession" );
	String parBack = request.getParameter( "cBack" );

	// Set login url for the case the user timed out and need to re-login
	StringBuffer regDest = new StringBuffer();
	if ( parStudy == null ) {
		regDest.append( parBack == null ? "/cases/index.jsp" : parBack );
	}
	else {
		regDest.append( "dotcom/parseStudyData.jsp?cStudy=" );
		regDest.append( Helper.URLEncodeUTF8( parStudy ) );
		if ( parSession != null ) {
			regDest.append( "&cSession=" ).append( Helper.URLEncodeUTF8( parSession ) );
		}
		if ( parBack != null ) {
			regDest.append( "&cBack=" ).append( Helper.URLEncodeUTF8( parBack ) );
		}
	}

	is.regDest = regDest.toString();
	if ( !is.regOK() ) {
		return;
	}

	if ( null == parBack )
		parBack = request.getHeader( "Referer" );
	if ( null == parBack )
		parBack = "javascript:self.history.back();";

	// Checking user rights
	if ( is.regRights == null || ( is.regRights.indexOf( "@" ) == -1 && is.regRights.indexOf( "A" ) == -1 ) ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "NO_RIGHTS_REQ_PAGE" ), m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}

	if ( parStudy == null ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "GENERAL_MISSING_PARAMETER" ) + ": cStudy", m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}

	if ( parSession == null ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "GENERAL_MISSING_PARAMETER" ) + ": cSession", m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}

	StudyComparator sc = new StudyComparator( parSession, parStudy );
	if ( !sc.collectStudyData( is, true ) ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_STUDY_CHANGED_HUB" ), m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}

	int numDevs = 0;
	try {
		numDevs = Integer.parseInt( request.getParameter( "cNumDevs" ) );
	}
	catch ( Throwable th ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "GENERAL_MISSING_OR_INVALID_PARAMETER" ) + ": cNumDevs", m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}

	String devices[] = sc.getDevices( is );
	if ( devices.length != numDevs ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_STUDY_CHANGED_HUB" ) + " (" + sc.getDevices( is ).length + " vs. " + numDevs + ")", m_tabPage, LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_ERROR_HUB" ), parBack );
	}

	// Processing other parameters
	String target = request.getParameter( "cTarget" );
	if ( target == null ) {
		throw new IMGeneralException( LocalizedMessage.getSimpleMessage( "GENERAL_MISSING_PARAMETER" ) + ": cTarget", m_tabPage, LocalizedMessage.getSimpleMessage( "GENERAL_ERROR_TITLE" ), parBack );
	}


	String imagesHubs[] = request.getParameterValues( "cKeepImages" );
	String reportHubs[] = request.getParameterValues( "cKeepReport" );

	List<String> keepImagesHubs = Arrays.asList( imagesHubs == null ? new String[ 0 ] : imagesHubs );
	List<String> keepReportHubs = Arrays.asList( reportHubs == null ? new String[ 0 ] : reportHubs );

	Set<String> keepAnyHubs = new HashSet<String>();
	keepAnyHubs.addAll( keepImagesHubs );
	keepAnyHubs.addAll( keepReportHubs );

	boolean isConfirmPage = !"correct".equals( request.getParameter( "cOp" ) );

%><HTML><HEAD><TITLE><%= LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_HTMLPAGE_TITLE" ) %></title>
<%
	TabToolbox.writeIMpopup(out);
%>
<SCRIPT language="JavaScript">

	var needUnlock = 1; // in default unlock study when leaving page

	function unlockStudy() {
		if (needUnlock) {
			jsPopup('unlockMultiple.jsp?cStudy=<%=parStudy%>&cSession=<%=parSession%>', 'unlockwindow',false);
		}
	}	

	function formSubmit(frm) {
		needUnlock = 0;
		frm.submit();
	}

<%
	if (!isConfirmPage) {
%>
	function getJSElementByID(id) {
		if (document.getElementById)
			return document.getElementById(id);
		else if (document.all)
			return document.all[id];
		else if (document.layers)
			return document.layers[id];
		else
			alert("Browser not compatible"); //TODO
	}

	function changeStyleColor(node, color) {
		var element = getJSElementByID(node);
		element.style.color = color;
	}

	function writeTD(node, text) {
		var element = getJSElementByID(node);
		element.innerHTML = text;
	}
		
<%
	}
%>
</SCRIPT>
<%
	imagemed.tables.Table.printCommonScript( is );
	imagemed.http.tabs.TabBean.writeStyleOnly( is );
%>
</head>
<BODY onUnload='unlockStudy()'>
<%

	out.println( "<!--" );
	for ( Enumeration<?> e = request.getParameterNames(); e.hasMoreElements(); ) {
		String name = (String) e.nextElement();
		String values[] = request.getParameterValues( name );
		out.println( name + ":" );
		for ( int i = 0; i < values.length; i++ ) {
			out.println( "\t[" + values[ i ] + "]" );
		}
	}
	out.println( "-->" );


	TabToolbox.makeTabs( is, is.tabbean, m_tabPage, "White" );

	Vector<Properties> vLock = new Vector<Properties>();
	if ( !sc.lock( "CheckLock", is._sid, false, is, vLock ) ) {
		// Lock has been stolen
		StringBuffer msg = new StringBuffer();
		msg.append( "<H2 style='text-align: center;'>" );
		for ( Iterator<Properties> iter = vLock.iterator(); iter.hasNext(); ) {
			Properties prpLock = iter.next();
			String device = prpLock.getProperty( "DEVICE" );
			String lockerName = prpLock.getProperty( "IMLockOwner" );
			String lockAge = prpLock.getProperty( "IMLockAge" );
			lockerName = ( null == lockerName ) ? LocalizedMessage.getSimpleMessage( "GENERIC_ANOTHER_USER" ) : IMTypePN.displayOrdinaryText( lockerName );

			msg.append( "<BR>" + LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_STUDY_LOCK_STOLEN", new Format[]{ null, null, null }, new Object[]{ device, lockerName, ReportToolbox.getLockAgeMessage( lockAge ) }, false ) + ".<P>\n" );
		}
		msg.append( "</H2>" );

		MessageBox box = new MessageBox( LocalizedMessage.getSimpleMessage( "CORRECTION_TOOL_LOCK_ERROR_TITLE" ), "60%" );
		box.setMessage( msg.toString() );
		box.addButton( new Button( parBack, "BUTTON_BACK", "BUTTON_BACK_TITLE" ) );
%><BR><CENTER><%
		box.print( is.out );
%>
</CENTER></BODY></HTML>
<%
		return;
	}


	if ( !isConfirmPage ) {
		// DICOM objects exist on the HUB the study to be reside on
		Set<ObjectData> images = sc.getImages( target );

		// DICOM objects do not exist on the HUB the study to be reside on
//		TreeMap missingImages = sc.getImagesMissing(target);
		int i = 0;

		// print to-do table
%>
 <BR><BR>
 <CENTER><table width="50%" border=1 cellpadding=2 cellspacing=2><tr class=BgShadow><td>
 <table width="100%" border=0 style="color: gray"><COL><COL align=center width="20%">
 <%
	 	out.println( "<tr id=\"trlock\"><td>" + LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_SYSTEM_LOCK_STUDIES_HUB" ) + "</td><td width=\"10%\" id=\"tdlock\">&nbsp;</td></tr>" );
 		for ( i = 0; i < devices.length; i++ ) {
 			if ( target.equals( devices[ i ] ) ) {
 				continue;
 			}

 			if ( !keepAnyHubs.contains( devices[ i ] ) ) {
 				// No object to be kept from HUB
 				continue;
 			}

 			out.println( "<tr id=\"trgetobj_" + devices[ i ] + "\"><td>" + LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_GET_OBJECTS_FROM_DEVICE", new Format[]{ null }, new Object[]{ devices[ i ] }, false ) +
 					"</td><td id=\"tdgetobj_" + devices[ i ] + "\">&nbsp;</td></tr>" );
 		}

 		out.println( "<tr id=\"trsetowner\"><td>" + LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_SET_TARGET_STUDY_OWNER", new Format[]{ null }, new Object[]{ target }, false ) + "</td><td id=\"tdsetowner\">&nbsp;</td></tr>" );

 		String putProcessMsg = keepAnyHubs.size() > 0 ?
 				LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_PUT_IMAGES_TARGET", new Format[]{ null }, new Object[]{ target }, false ) :
 				LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_FIX_STUDY_AT_TARGET", new Format[]{ null }, new Object[]{ target }, false );
 		out.println( "<tr id=\"trputprocess\"><td>" + putProcessMsg + "</td><td id=\"tdputprocess\">&nbsp;</td></tr>" );

 		out.println( "<tr id=\"trobsolete\"><td>" + LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_SET_STUDY_OBSOLETE_EXCEPT_TARGET", new Format[]{ null }, new Object[]{ target }, false ) + "</td><td id=\"tdobsolete\">&nbsp;</td></tr>" );

 		out.println( "<tr id=\"trunlock\"><td>" + LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_UNLOCK_STUDIES_HUB" ) + "</td><td id=\"tdunlock\">&nbsp;</td></tr>" );
 %>
 </table></tr></td></table><BR><BR><BR>
<%
	// start the correction
		out.flush();

		// System Lock studies
		CorrectionTool.nextStep( is.out, "trlock" );
		if ( !CorrectionTool.checkResult( is.out, CorrectionTool.systemLock( sc, is ), parBack, "tdlock" ) ) {
			return;
		}

		// Create a flag file that cleanup is in progress. The flag file will be deleted
		//   - by the cleanup tool if fails
		//   - by syncStudy.jsp when the delete from the hub(s) gets propagated back
		CorrectionTool.createInProgressFlag( parStudy );


		// Get object data to be prepared from HUBs the study will be removed from

		// multipart request to be sent to the HUB the study will remain on
		MultipartRequest mpr = new MultipartRequest();

		mpr.addParam( "cStudy", parStudy, MultipartRequest.MP_STRING_PARAM );
		mpr.addParam( "cSession", parSession, MultipartRequest.MP_STRING_PARAM );
		if ( is.regID != null ) {
			mpr.addParam( "cUser", is.regID, MultipartRequest.MP_STRING_PARAM );
		}

		int numTarFiles = 0;
		for ( i = 0; i < devices.length; i++ ) {
			if ( target.equals( devices[ i ] ) ) {
				continue;
			}

			if ( !keepAnyHubs.contains( devices[ i ] ) ) {
				// No object to be kept from HUB
				continue;
			}

			boolean copyImages = keepImagesHubs.contains( devices[ i ] );
			boolean copyReports = keepReportHubs.contains( devices[ i ] );

			CorrectionTool.nextStep( is.out, "trgetobj_" + devices[ i ] );

			File dataFile = new File( IMEnvBase.getEnv( "TMP" ) + "/multihub." + parSession + "." + devices[ i ] + ".tar.gz" );
			MessageBox mb = CorrectionTool.getObjectsFrom( dataFile, parStudy, parSession, devices[ i ], copyImages, copyReports, images, is.regID, target );

			if ( !CorrectionTool.checkResult( is.out, mb, parBack, "tdgetobj_" + devices[ i ] ) ) {
				CorrectionTool.removeInProgressFlag( parStudy );
				return;
			}

			if ( dataFile.length() != 0 ) {
				mpr.addParam( "cHub" + numTarFiles, devices[ i ], MultipartRequest.MP_STRING_PARAM );
				mpr.addParam( "cFile" + numTarFiles, dataFile.getAbsolutePath(), MultipartRequest.MP_FILE_PARAM );
				numTarFiles++;
			}

		}

		mpr.addParam( "cNumFiles", "" + numTarFiles, MultipartRequest.MP_STRING_PARAM );
		mpr.addParam( "cDelReport", "" + ( !keepReportHubs.contains( target ) ), MultipartRequest.MP_STRING_PARAM );

		// Set target device to be the owner
		CorrectionTool.nextStep( is.out, "trsetowner" );
		if ( !CorrectionTool.checkResult( is.out, CorrectionTool.setOwnerTo( is, target, parStudy ), parBack, "tdsetowner" ) ) {
			CorrectionTool.removeInProgressFlag( parStudy );
			return;
		}

		// Put object data to the HUB the study will reside on
		CorrectionTool.nextStep( is.out, "trputprocess" );
		if ( !CorrectionTool.checkResult( is.out, CorrectionTool.putObjectsTo( is.out, target, parSession, mpr ), parBack, "tdputprocess" ) ) {
			CorrectionTool.removeInProgressFlag( parStudy );
			return;
		}

		// Remove study from other HUBs
		CorrectionTool.nextStep( is.out, "trobsolete" );
		if ( !CorrectionTool.checkResult( is.out, CorrectionTool.obsoleteStudy( is, parStudy, parSession, keepAnyHubs, is.regID, target ),
				parBack, "tdobsolete" ) ) {
			CorrectionTool.removeInProgressFlag( parStudy );
			return;
		}

		// Release System lock
		CorrectionTool.nextStep( is.out, "trunlock" );
		sc.lock( "Unlock", is._sid, true, is, vLock );
		CorrectionTool.checkResult( is.out, null, null, "tdunlock" );

		MessageBox mb = new MessageBox( LocalizedMessage.getSimpleMessage( "GENERIC_DONE" ), "60%" );
		mb.setMessage( "<H2 style='text-align: center;'><BR>" + LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_STUDY_SUCCESS_CORRECT" ) +
				".<BR><BR></H2><H3 style='text-align: center;'>" + LocalizedMessage.getSimpleMessage( "DOTCOM_CORRECT_STUDY_SUCCESS_NOTICE" ) + ".<BR></H3>" );

		mb.addButton( new Button( parBack, "BUTTON_BACK", "BUTTON_BACK_TITLE" ) );
		mb.print( is.out );
%>
<SCRIPT language="JavaScript">
	needUnlock = 0; 
</SCRIPT>
</BODY></HTML><%
		return;
	}
%>
<FORM METHOD="POST" NAME="correctionForm">
<%
	for ( Enumeration<?> e = request.getParameterNames(); e.hasMoreElements(); ) {
		String name = (String) e.nextElement();
%><INPUT TYPE="HIDDEN" NAME="<%= name %>" VALUE="<%= Helper.htmlEncode(request.getParameter(name)) %>">
<%
	}
%>
<INPUT TYPE="HIDDEN" NAME="cOp" VALUE="correct">
</FORM>
<BR>
<CENTER>
<%

	Vector<String> sourceHubs = new Vector<String>( devices.length - 1 );
	for ( int i = 0; i < devices.length; i++ ) {
		if ( target.equals( devices[ i ] ) ) {
			continue;
		}

		sourceHubs.add( devices[ i ] );
	}

	String sourceHubList = LocalizedMessage.getListAsLocalizedString( sourceHubs );

	StringBuffer msg = new StringBuffer();
	String applyLabel =  LocalizedMessage.getSimpleMessage( "BUTTON_APPLY" );
	msg.append( LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_CONFIRM_MESSAGE", new Format[] { null, null, null }, new Object[] { sourceHubList, target, applyLabel }, true ) );
	msg.append( "<BR>" );

	if (reportHubs != null) {
		boolean hasTargetReport = false;
		boolean hasOtherReport = false;
		
		for ( int i = 0; i < reportHubs.length; i++ ) {
			if ( target.equals( reportHubs[ i ] ) ) {
				hasTargetReport = true;
				continue;
			}

			hasOtherReport = true;
		}

		if ( hasTargetReport && hasOtherReport ) {
			msg.append( LocalizedMessage.getCompoundMessage( "DOTCOM_CORRECT_CONFIRM_MESSAGE_REPORTS", new Format[] { null }, new Object[] { target }, true ) );
		}
	}

	MessageBox messageBox = new MessageBox( LocalizedMessage.getSimpleMessage( "DOCTOM_CORRECT_STUDY_RESIDE_MULTIPLE_HUB" ), "60%");
	messageBox.setMessage( "<BR><div style='text-align: center;'>" + msg + "</div><BR>" );
	messageBox.addButton( new Button( "javascript:formSubmit(self.document.correctionForm)", "BUTTON_APPLY", "BUTTON_APPLY_STUDY_TITLE" ) );
	messageBox.addButton( new Button( parBack, "BUTTON_CANCEL", "BUTTON_CANCEL" ) );

	messageBox.print( is.out );
%>
</CENTER></BODY></HTML>
